/* check.c
 * 
 *  check for open socks server
 */

#include <strings.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netdb.h>
#include <pwd.h>
#include <sys/time.h>
#include "extern.h"
#include "config.h"
#include "defines.h"

int debug;
int mode=1;
int verbose=0;
int db=0;
int port=0;

struct passwd *userinfo;

	/*
	 * Not run as CGI so Parse command line params:
	 * -h <hostname>
	 * -v verbose mode (verbose about connections and results - no debug info)  [optional]
	 * -p <port> [optional]
	 * -a paranoid mode (reject if socks found) [optional]
	 * -r relaxed mode (reject on open socks only) [optional]
	 * -n normal mode (default, reject on authenticated or open proxy) [optional]
	 * -w write to db mode (write ip address to db based upon detection level) [optional]
	 * -d <debug level> [optional]
	 */

main(int argc, char **argv) {

	int i;
	int sock=0;

	char *hostname;

#ifdef DEBUG
	 debug=5;
#else
	 debug=0;
#endif
	if (!parse_config(argc, argv)) usage(argv[0]);
	
	if (debug) {
		printf("Debug Level=%d\n", debug);
		print_debug(2, "Hostname: %s\n", config->hostname?config->hostname:"not specified");
		print_debug(2, "Ports: 1080, 3128, 808%s%d\n", port?"0, ":"", port);
		print_debug(2, "Mode: %s\n", mode?mode==2?"Relaxed":"Normal":"Paranoid");
		print_debug(2, "DB creation: %s\n", db?"on":"off");
		print_config();
	}

	if (!config->hostname) usage(argv[0]);

	/* Get uid of this process, and set userinfo pw entry */
	get_userinfo();

	for(i=0;config->socks_ports[i];i++) {
		if (test_sock(config->hostname, config->socks_ports[i])) reject(config->exitval->socks);
	}
	for(i=0;config->http_ports[i];i++) {
		if (test_proxy(config->hostname, config->http_ports[i])) reject(config->exitval->http);
	}
	if (test_relay(config->hostname)) reject(config->exitval->relay); 

	cleanup();
	
}

void get_userinfo(void) {

	userinfo=getpwuid(geteuid());
	if (!userinfo) {
		perror("Unable to get userinfo");
		exit (1);
	}
}

int usage(char *progname) {
	fprintf(stderr, "Usage: %s [-v] [-r] [-n] [-a] [-w] -h <hostname> [-p <port>]\r\n", progname);
	exit(1);
}

void reject(int exit_val) {

	int t;
	unsigned char *i;

	i=getip(config->hostname);
	for(t=0; t<4; t++, i++) {
		printf("%hhi%s", *i, t<3?".":"");
	}
	printf(": rejected after tests\n");
	cleanup();
	exit(exit_val);

}

char *unescape_string(char *s) {

	char *p, *pt, *t, buf[3];
	int l, ret;

	p=s;
	l=strlen(s);
	l++;
	pt=t=calloc(sizeof(char), l);

	while (*s) {
		if (*s == '%') {
			s++; // found % we want the next 2 chars
			buf[0]=*s++;
			buf[1]=*s++;
			buf[2]='\0';
			ret=(int) strtol(buf, NULL, 16);
			*t++=ret;
		} else {
			*t++=*s++;
		}
	}
	t=pt;
	s=p;
	while (*t) *s++=*t++;
	*s='\0';
	free(pt);
}
